@@ -754,7 +754,7 @@ HANDLE WINAPI FindFirstFileExW( LPCWSTR filename, FINDEX_INFO_LEVELS level,
 
     status = NtOpenFile( &info->handle, GENERIC_READ | SYNCHRONIZE, &attr, &io,
                          FILE_SHARE_READ | FILE_SHARE_WRITE,
-                         FILE_DIRECTORY_FILE | FILE_SYNCHRONOUS_IO_NONALERT );
+                         FILE_DIRECTORY_FILE | FILE_SYNCHRONOUS_IO_NONALERT | FILE_OPEN_FOR_BACKUP_INTENT );
 
     if (status != STATUS_SUCCESS)
     {
@@ -801,13 +801,7 @@ HANDLE WINAPI FindFirstFileExW( LPCWSTR filename, FINDEX_INFO_LEVELS level,
         }
 
         info->data_len = io.Information;
-        if (!has_wildcard || info->data_len < info->data_size - max_entry_size)
-        {
-            if (has_wildcard)  /* release unused buffer space */
-                HeapReAlloc( GetProcessHeap(), HEAP_REALLOC_IN_PLACE_ONLY,
-                             info, offsetof( FIND_FIRST_INFO, data[info->data_len] ));
-            info->data_size = 0;  /* we read everything */
-        }
+        if (!has_wildcard) info->data_size = 0;  /* we read everything */
 
         if (!FindNextFileW( info, data ))
         {
@@ -1166,6 +1160,7 @@ HFILE WINAPI OpenFile( LPCSTR name, OFSTRUCT *ofs, UINT mode )
     HANDLE handle;
     FILETIME filetime;
     WORD filedatetime[2];
+    DWORD len;
 
     if (!ofs) return HFILE_ERROR;
 
@@ -1201,7 +1196,13 @@ HFILE WINAPI OpenFile( LPCSTR name, OFSTRUCT *ofs, UINT mode )
     /* the watcom 10.6 IDE relies on a valid path returned in ofs->szPathName
        Are there any cases where getting the path here is wrong?
        Uwe Bonnes 1997 Apr 2 */
-    if (!GetFullPathNameA( name, sizeof(ofs->szPathName), ofs->szPathName, NULL )) goto error;
+    len = GetFullPathNameA( name, sizeof(ofs->szPathName), ofs->szPathName, NULL );
+    if (!len) goto error;
+    if (len >= sizeof(ofs->szPathName))
+    {
+        SetLastError(ERROR_INVALID_DATA);
+        goto error;
+    }
 
     /* OF_PARSE simply fills the structure */
 
@@ -1224,8 +1225,13 @@ HFILE WINAPI OpenFile( LPCSTR name, OFSTRUCT *ofs, UINT mode )
     {
         /* Now look for the file */
 
-        if (!SearchPathA( NULL, name, NULL, sizeof(ofs->szPathName), ofs->szPathName, NULL ))
+        len = SearchPathA( NULL, name, NULL, sizeof(ofs->szPathName), ofs->szPathName, NULL );
+        if (!len) goto error;
+        if (len >= sizeof(ofs->szPathName))
+        {
+            SetLastError(ERROR_INVALID_DATA);
             goto error;
+        }
 
         TRACE("found %s\n", debugstr_a(ofs->szPathName) );
 
