@@ -85,8 +85,8 @@ static BOOL process_attach( HMODULE module )
 
     NtQuerySystemInformation( SystemBasicInformation, &system_info, sizeof(system_info), NULL );
 
-    /* Setup registry locale information */
-    LOCALE_InitRegistry();
+    /* Setup registry timezone information */
+    TIMEZONE_InitRegistry();
 
     /* Setup computer name */
     COMPUTERNAME_Init();
@@ -175,38 +175,6 @@ INT WINAPI MulDiv( INT nMultiplicand, INT nMultiplier, INT nDivisor)
     return ret;
 }
 
-
-/******************************************************************************
- *           GetTickCount64       (KERNEL32.@)
- */
-ULONGLONG WINAPI DECLSPEC_HOTPATCH GetTickCount64(void)
-{
-    LARGE_INTEGER counter, frequency;
-
-    NtQueryPerformanceCounter( &counter, &frequency );
-    return counter.QuadPart * 1000 / frequency.QuadPart;
-}
-
-
-/***********************************************************************
- *           GetTickCount       (KERNEL32.@)
- *
- * Get the number of milliseconds the system has been running.
- *
- * PARAMS
- *  None.
- *
- * RETURNS
- *  The current tick count.
- *
- * NOTES
- *  The value returned will wrap around every 2^32 milliseconds.
- */
-DWORD WINAPI DECLSPEC_HOTPATCH GetTickCount(void)
-{
-    return GetTickCount64();
-}
-
 /******************************************************************************
  *           GetSystemRegistryQuota       (KERNEL32.@)
  */
