@@ -27,10 +27,13 @@
 extern WCHAR *file_name_AtoW( LPCSTR name, BOOL alloc ) DECLSPEC_HIDDEN;
 extern DWORD file_name_WtoA( LPCWSTR src, INT srclen, LPSTR dest, INT destlen ) DECLSPEC_HIDDEN;
 extern void init_startup_info( RTL_USER_PROCESS_PARAMETERS *params ) DECLSPEC_HIDDEN;
+extern void init_locale(void) DECLSPEC_HIDDEN;
 
 extern const WCHAR windows_dir[] DECLSPEC_HIDDEN;
 extern const WCHAR system_dir[] DECLSPEC_HIDDEN;
-extern HANDLE kernel32_handle DECLSPEC_HIDDEN;
+
+static const BOOL is_win64 = (sizeof(void *) > sizeof(int));
+extern BOOL is_wow64 DECLSPEC_HIDDEN;
 
 static inline BOOL is_console_handle(HANDLE h)
 {
