@@ -134,6 +134,8 @@ static int (__cdecl *p__fpieee_flt)(ULONG, EXCEPTION_POINTERS*, int (__cdecl *ha
 static int (__cdecl *p__memicmp)(const char*, const char*, size_t);
 static int (__cdecl *p__memicmp_l)(const char*, const char*, size_t, _locale_t);
 static int (__cdecl *p__vsnwprintf)(wchar_t *buffer,size_t count, const wchar_t *format, __ms_va_list valist);
+static size_t (__cdecl *p___strncnt)(const char *str, size_t count);
+static int (__cdecl *p_swscanf)(const wchar_t *str, const wchar_t* format, ...);
 
 /* make sure we use the correct errno */
 #undef errno
@@ -403,6 +405,8 @@ static BOOL init(void)
     SET(p__memicmp, "_memicmp");
     SET(p__memicmp_l, "_memicmp_l");
     SET(p__vsnwprintf, "_vsnwprintf");
+    SET(p___strncnt, "__strncnt");
+    SET(p_swscanf, "swscanf");
 
     if (sizeof(void *) == 8)
     {
@@ -900,7 +904,7 @@ static void test_bsearch_s(void)
     CHECK_CALLED(invalid_parameter_handler, EINVAL);
 
     /* just try all array sizes */
-    for (j=1;j<sizeof(arr)/sizeof(arr[0]);j++) {
+    for (j=1; j<ARRAY_SIZE(arr); j++) {
         for (i=0;i<j;i++) {
             l = arr[i];
             g_bsearch_s_context_counter = 0;
@@ -1494,7 +1498,7 @@ static void test__AdjustPointer(void)
     void *ret;
     int i;
 
-    for(i=0; i<sizeof(data)/sizeof(data[0]); i++) {
+    for(i=0; i<ARRAY_SIZE(data); i++) {
         ret = p__AdjustPointer(data[i].ptr, &data[i].this_ptr_offsets);
         ok(ret == data[i].ret, "%d) __AdjustPointer returned %p, expected %p\n", i, ret, data[i].ret);
     }
@@ -1734,7 +1738,7 @@ static void test__fpieee_flt(void)
     ok(ret == EXCEPTION_CONTINUE_SEARCH, "_fpieee_flt returned %d\n", ret);
     ok(handler_called == 0, "handler_called = %d\n", handler_called);
 
-    for(i=0; i<sizeof(test_data)/sizeof(test_data[0]); i++) {
+    for(i=0; i<ARRAY_SIZE(test_data); i++) {
         ep.ExceptionRecord = &rec;
         ep.ContextRecord = &ctx;
         memset(&rec, 0, sizeof(rec));
@@ -1894,6 +1898,46 @@ static void test__vsnwprintf(void)
     ok(p_set_invalid_parameter_handler(old_handler) == test_invalid_parameter_handler, "Cannot reset invalid parameter handler\n");
 }
 
+static void test___strncnt(void)
+{
+    static const struct
+    {
+        const char *str;
+        size_t size;
+        size_t ret;
+    }
+    strncnt_tests[] =
+    {
+        { NULL, 0, 0 },
+        { "a", 0, 0 },
+        { "a", 1, 1 },
+        { "a", 10, 1 },
+        { "abc", 1, 1 },
+    };
+    unsigned int i;
+    size_t ret;
+
+    if (0)
+        ret = p___strncnt(NULL, 1);
+
+    for (i = 0; i < ARRAY_SIZE(strncnt_tests); ++i)
+    {
+        ret = p___strncnt(strncnt_tests[i].str, strncnt_tests[i].size);
+        ok(ret == strncnt_tests[i].ret, "%u: unexpected return value %u.\n", i, (int)ret);
+    }
+}
+
+static void test_swscanf(void)
+{
+
+    wchar_t buffer[100];
+    int ret;
+
+    /* check WEOF */
+    ret = p_swscanf(L" \t\n\n", L"%s", buffer);
+    ok( ret == (short)WEOF, "ret = %d\n", ret );
+}
+
 START_TEST(msvcr90)
 {
     if(!init())
@@ -1931,4 +1975,6 @@ START_TEST(msvcr90)
 #ifdef __i386__
     test__fpieee_flt();
 #endif
+    test___strncnt();
+    test_swscanf();
 }
