@@ -35,7 +35,6 @@
 #include "resource.h"
 
 #include "wine/debug.h"
-#include "wine/unicode.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(ole);
 
@@ -182,14 +181,14 @@ static void get_descriptors(HWND hdlg, ps_struct_t *ps_struct)
     if(ps_struct->source_name == NULL && ps_struct->link_source_name == NULL)
     {
         WCHAR buf[200];
-        LoadStringW(OLEDLG_hInstance, IDS_PS_UNKNOWN_SRC, buf, sizeof(buf)/sizeof(WCHAR));
+        LoadStringW(OLEDLG_hInstance, IDS_PS_UNKNOWN_SRC, buf, ARRAY_SIZE(buf));
         ps_struct->source_name = strdupW(buf);
     }
 
     if(ps_struct->type_name == NULL && ps_struct->link_type_name == NULL)
     {
         WCHAR buf[200];
-        LoadStringW(OLEDLG_hInstance, IDS_PS_UNKNOWN_TYPE, buf, sizeof(buf)/sizeof(WCHAR));
+        LoadStringW(OLEDLG_hInstance, IDS_PS_UNKNOWN_TYPE, buf, ARRAY_SIZE(buf));
         ps_struct->type_name = strdupW(buf);
     }
 }
@@ -228,7 +227,7 @@ static DWORD init_pastelist(HWND hdlg, OLEUIPASTESPECIALW *ps)
     }
 
     /* The native version grabs only the first 20 fmts and we do the same */
-    hr = IEnumFORMATETC_Next(penum, sizeof(fmts)/sizeof(fmts[0]), fmts, &fetched);
+    hr = IEnumFORMATETC_Next(penum, ARRAY_SIZE(fmts), fmts, &fetched);
     TRACE("got %d formats hr %08x\n", fetched, hr);
 
     if(SUCCEEDED(hr))
@@ -431,16 +430,16 @@ static void update_result_text(HWND hdlg, const ps_struct_t *ps_struct)
             res_id = IDS_PS_PASTE_LINK_DATA;
     }
 
-    LoadStringW(OLEDLG_hInstance, res_id, resource_txt, sizeof(resource_txt)/sizeof(WCHAR));
-    if((ptr = strstrW(resource_txt, percent_s)))
+    LoadStringW(OLEDLG_hInstance, res_id, resource_txt, ARRAY_SIZE(resource_txt));
+    if((ptr = wcsstr(resource_txt, percent_s)))
     {
         /* FIXME handle %s in ResultText. Sub appname if IDS_PS_PASTE_OBJECT{_AS_ICON}.  Else sub appropriate type name */
-        size_t result_txt_len = strlenW(pent->lpstrResultText);
+        size_t result_txt_len = lstrlenW(pent->lpstrResultText);
         ptrdiff_t offs = (char*)ptr - (char*)resource_txt;
-        result_txt = HeapAlloc(GetProcessHeap(), 0, (strlenW(resource_txt) + result_txt_len - 1) * sizeof(WCHAR));
+        result_txt = HeapAlloc(GetProcessHeap(), 0, (lstrlenW(resource_txt) + result_txt_len - 1) * sizeof(WCHAR));
         memcpy(result_txt, resource_txt, offs);
         memcpy((char*)result_txt + offs, pent->lpstrResultText, result_txt_len * sizeof(WCHAR));
-        memcpy((char*)result_txt + offs + result_txt_len * sizeof(WCHAR), ptr + 2, (strlenW(ptr + 2) + 1) * sizeof(WCHAR));
+        memcpy((char*)result_txt + offs + result_txt_len * sizeof(WCHAR), ptr + 2, (lstrlenW(ptr + 2) + 1) * sizeof(WCHAR));
     }
     else
         result_txt = resource_txt;
