@@ -82,14 +82,14 @@ static inline VfwCapture *impl_from_IPersistPropertyBag(IPersistPropertyBag *ifa
     return CONTAINING_RECORD(iface, VfwCapture, IPersistPropertyBag_iface);
 }
 
-static IPin *vfw_capture_get_pin(struct strmbase_filter *iface, unsigned int index)
+static struct strmbase_pin *vfw_capture_get_pin(struct strmbase_filter *iface, unsigned int index)
 {
     VfwCapture *This = impl_from_strmbase_filter(iface);
 
     if (index >= 1)
         return NULL;
 
-    return &This->source.pin.IPin_iface;
+    return &This->source.pin;
 }
 
 static void vfw_capture_destroy(struct strmbase_filter *iface)
@@ -205,7 +205,8 @@ AMStreamConfig_SetFormat(IAMStreamConfig *iface, AM_MEDIA_TYPE *pmt)
     HRESULT hr;
     VfwCapture *This = impl_from_IAMStreamConfig(iface);
 
-    TRACE("(%p): %p->%p\n", iface, pmt, pmt ? pmt->pbFormat : NULL);
+    TRACE("filter %p, mt %p.\n", This, pmt);
+    strmbase_dump_media_type(pmt);
 
     if (This->filter.state != State_Stopped)
     {
@@ -219,8 +220,6 @@ AMStreamConfig_SetFormat(IAMStreamConfig *iface, AM_MEDIA_TYPE *pmt)
         return E_POINTER;
     }
 
-    dump_AM_MEDIA_TYPE(pmt);
-
     if (This->source.pin.peer)
     {
         hr = IPin_QueryAccept(This->source.pin.peer, pmt);
@@ -244,9 +243,13 @@ static HRESULT WINAPI
 AMStreamConfig_GetFormat( IAMStreamConfig *iface, AM_MEDIA_TYPE **pmt )
 {
     VfwCapture *This = impl_from_IAMStreamConfig(iface);
+    HRESULT hr;
 
     TRACE("%p -> (%p)\n", iface, pmt);
-    return qcap_driver_get_format(This->driver_info, pmt);
+    hr = qcap_driver_get_format(This->driver_info, pmt);
+    if (SUCCEEDED(hr))
+        strmbase_dump_media_type(*pmt);
+    return hr;
 }
 
 static HRESULT WINAPI
