@@ -82,6 +82,85 @@ static inline BOOL is_started(VBScript *This)
 
 static HRESULT exec_global_code(script_ctx_t *ctx, vbscode_t *code, VARIANT *res)
 {
+    function_t *func_iter, **new_funcs;
+    dynamic_var_t *var, **new_vars;
+    size_t cnt, i;
+
+    cnt = ctx->global_vars_cnt + code->main_code.var_cnt;
+    if (cnt > ctx->global_vars_size)
+    {
+        if (ctx->global_vars)
+            new_vars = heap_realloc(ctx->global_vars, cnt * sizeof(*new_vars));
+        else
+            new_vars = heap_alloc(cnt * sizeof(*new_vars));
+        if (!new_vars)
+            return E_OUTOFMEMORY;
+        ctx->global_vars = new_vars;
+        ctx->global_vars_size = cnt;
+    }
+
+    cnt = ctx->global_funcs_cnt;
+    for (func_iter = code->funcs; func_iter; func_iter = func_iter->next)
+        cnt++;
+    if (cnt > ctx->global_funcs_size)
+    {
+        if (ctx->global_funcs)
+            new_funcs = heap_realloc(ctx->global_funcs, cnt * sizeof(*new_funcs));
+        else
+            new_funcs = heap_alloc(cnt * sizeof(*new_funcs));
+        if (!new_funcs)
+            return E_OUTOFMEMORY;
+        ctx->global_funcs = new_funcs;
+        ctx->global_funcs_size = cnt;
+    }
+
+    for (i = 0; i < code->main_code.var_cnt; i++)
+    {
+        if (!(var = heap_pool_alloc(&ctx->heap, sizeof(*var))))
+            return E_OUTOFMEMORY;
+
+        var->name = code->main_code.vars[i].name;
+        V_VT(&var->v) = VT_EMPTY;
+        var->is_const = FALSE;
+        var->array = NULL;
+
+        ctx->global_vars[ctx->global_vars_cnt + i] = var;
+    }
+
+    ctx->global_vars_cnt += code->main_code.var_cnt;
+
+    for (func_iter = code->funcs; func_iter; func_iter = func_iter->next)
+    {
+        for (i = 0; i < ctx->global_funcs_cnt; i++)
+        {
+            if (!wcsicmp(ctx->global_funcs[i]->name, func_iter->name))
+            {
+                /* global function already exists, replace it */
+                ctx->global_funcs[i] = func_iter;
+                break;
+            }
+        }
+        if (i == ctx->global_funcs_cnt)
+            ctx->global_funcs[ctx->global_funcs_cnt++] = func_iter;
+    }
+
+    if (code->classes)
+    {
+        class_desc_t *class = code->classes;
+
+        while (1)
+        {
+            class->ctx = ctx;
+            if (!class->next)
+                break;
+            class = class->next;
+        }
+
+        class->next = ctx->classes;
+        ctx->classes = code->classes;
+        code->last_class = class;
+    }
+
     code->pending_exec = FALSE;
     return exec_script(ctx, TRUE, &code->main_code, NULL, NULL, res);
 }
@@ -130,13 +209,35 @@ IDispatch *lookup_named_item(script_ctx_t *ctx, const WCHAR *name, unsigned flag
 
 static void release_script(script_ctx_t *ctx)
 {
+    vbscode_t *code, *code_next;
     class_desc_t *class_desc;
+    unsigned i;
 
     collect_objects(ctx);
     clear_ei(&ctx->ei);
 
-    release_dynamic_vars(ctx->global_vars);
+    for(i = 0; i < ctx->global_vars_cnt; i++)
+        release_dynamic_var(ctx->global_vars[i]);
+
+    heap_free(ctx->global_vars);
+    heap_free(ctx->global_funcs);
     ctx->global_vars = NULL;
+    ctx->global_vars_cnt = 0;
+    ctx->global_vars_size = 0;
+    ctx->global_funcs = NULL;
+    ctx->global_funcs_cnt = 0;
+    ctx->global_funcs_size = 0;
+
+    LIST_FOR_EACH_ENTRY_SAFE(code, code_next, &ctx->code_list, vbscode_t, entry)
+    {
+        if(code->is_persistent)
+        {
+            code->pending_exec = TRUE;
+            if(code->last_class) code->last_class->next = NULL;
+        }
+        else
+            release_vbscode(code);
+    }
 
     while(!list_empty(&ctx->named_items)) {
         named_item_t *iter = LIST_ENTRY(list_head(&ctx->named_items), named_item_t, entry);
@@ -178,18 +279,14 @@ static void release_script(script_ctx_t *ctx)
         IDispatchEx_Release(&script_obj->IDispatchEx_iface);
     }
 
-    detach_global_objects(ctx);
     heap_pool_free(&ctx->heap);
     heap_pool_init(&ctx->heap);
 }
 
-static void destroy_script(script_ctx_t *ctx)
+static void release_code_list(script_ctx_t *ctx)
 {
     while(!list_empty(&ctx->code_list))
         release_vbscode(LIST_ENTRY(list_head(&ctx->code_list), vbscode_t, entry));
-
-    release_script(ctx);
-    heap_free(ctx);
 }
 
 static void decrease_state(VBScript *This, SCRIPTSTATE state)
@@ -202,16 +299,17 @@ static void decrease_state(VBScript *This, SCRIPTSTATE state)
         /* FALLTHROUGH */
     case SCRIPTSTATE_STARTED:
     case SCRIPTSTATE_DISCONNECTED:
-        if(This->state == SCRIPTSTATE_DISCONNECTED)
-            change_state(This, SCRIPTSTATE_INITIALIZED);
-        if(state == SCRIPTSTATE_INITIALIZED)
-            break;
+        change_state(This, SCRIPTSTATE_INITIALIZED);
         /* FALLTHROUGH */
     case SCRIPTSTATE_INITIALIZED:
     case SCRIPTSTATE_UNINITIALIZED:
         change_state(This, state);
+        if(state == SCRIPTSTATE_INITIALIZED)
+            break;
         release_script(This->ctx);
         This->thread_id = 0;
+        if(state == SCRIPTSTATE_CLOSED)
+            release_code_list(This->ctx);
         break;
     case SCRIPTSTATE_CLOSED:
         break;
@@ -387,7 +485,8 @@ static ULONG WINAPI VBScript_Release(IActiveScript *iface)
 
     if(!ref) {
         decrease_state(This, SCRIPTSTATE_CLOSED);
-        destroy_script(This->ctx);
+        detach_global_objects(This->ctx);
+        heap_free(This->ctx);
         heap_free(This);
     }
 
@@ -452,7 +551,7 @@ static HRESULT WINAPI VBScript_SetScriptState(IActiveScript *iface, SCRIPTSTATE
         return S_OK;
     }
 
-    if(!This->is_initialized)
+    if(!This->is_initialized || !This->ctx->site)
         return E_UNEXPECTED;
 
     switch(ss) {
@@ -464,7 +563,7 @@ static HRESULT WINAPI VBScript_SetScriptState(IActiveScript *iface, SCRIPTSTATE
         exec_queued_code(This->ctx);
         break;
     case SCRIPTSTATE_INITIALIZED:
-        FIXME("unimplemented SCRIPTSTATE_INITIALIZED\n");
+        decrease_state(This, SCRIPTSTATE_INITIALIZED);
         return S_OK;
     case SCRIPTSTATE_DISCONNECTED:
         FIXME("unimplemented SCRIPTSTATE_DISCONNECTED\n");
@@ -574,7 +673,7 @@ static HRESULT WINAPI VBScript_GetScriptDispatch(IActiveScript *iface, LPCOLESTR
 {
     VBScript *This = impl_from_IActiveScript(iface);
 
-    TRACE("(%p)->(%p)\n", This, ppdisp);
+    TRACE("(%p)->(%s %p)\n", This, debugstr_w(pstrItemName), ppdisp);
 
     if(!ppdisp)
         return E_POINTER;
