@@ -1188,14 +1188,14 @@ static inline struct gstdemux_source *impl_from_IMediaSeeking(IMediaSeeking *ifa
     return CONTAINING_RECORD(iface, struct gstdemux_source, seek.IMediaSeeking_iface);
 }
 
-static IPin *gstdemux_get_pin(struct strmbase_filter *base, unsigned int index)
+static struct strmbase_pin *gstdemux_get_pin(struct strmbase_filter *base, unsigned int index)
 {
     struct gstdemux *filter = impl_from_strmbase_filter(base);
 
     if (!index)
-        return &filter->sink.IPin_iface;
+        return &filter->sink;
     else if (index <= filter->cStreams)
-        return &filter->ppPins[index - 1]->pin.pin.IPin_iface;
+        return &filter->ppPins[index - 1]->pin.pin;
     return NULL;
 }
 
@@ -1216,7 +1216,7 @@ static void gstdemux_destroy(struct strmbase_filter *iface)
         assert(hr == S_OK);
     }
 
-    FreeMediaType(&filter->sink.mtCurrent);
+    FreeMediaType(&filter->sink.mt);
     if (filter->alloc)
         IMemAllocator_Release(filter->alloc);
     filter->alloc = NULL;
@@ -1406,10 +1406,6 @@ static HRESULT WINAPI GST_Run(IBaseFilter *iface, REFERENCE_TIME tStart)
     if (!This->container)
         return VFW_E_NOT_CONNECTED;
 
-    EnterCriticalSection(&This->filter.csFilter);
-    This->filter.rtStreamStart = tStart;
-    LeaveCriticalSection(&This->filter.csFilter);
-
     gst_element_get_state(This->container, &now, NULL, -1);
     if (now == GST_STATE_PLAYING)
         return S_OK;
@@ -1423,7 +1419,6 @@ static HRESULT WINAPI GST_Run(IBaseFilter *iface, REFERENCE_TIME tStart)
 
     EnterCriticalSection(&This->filter.csFilter);
     gst_element_set_state(This->container, GST_STATE_PLAYING);
-    This->filter.rtStreamStart = tStart;
 
     for (i = 0; i < This->cStreams; i++) {
         hr = BaseOutputPinImpl_Active(&This->ppPins[i]->pin);
@@ -1981,7 +1976,7 @@ static HRESULT WINAPI GSTInPin_ReceiveConnection(IPin *iface, IPin *pReceivePin,
     HRESULT hr = S_OK;
 
     TRACE("filter %p, peer %p, mt %p.\n", filter, pReceivePin, pmt);
-    dump_AM_MEDIA_TYPE(pmt);
+    strmbase_dump_media_type(pmt);
 
     mark_wine_thread();
 
@@ -2027,7 +2022,7 @@ static HRESULT WINAPI GSTInPin_ReceiveConnection(IPin *iface, IPin *pReceivePin,
         if (pAlloc)
             IMemAllocator_Release(pAlloc);
         if (SUCCEEDED(hr)) {
-            CopyMediaType(&filter->sink.mtCurrent, pmt);
+            CopyMediaType(&filter->sink.mt, pmt);
             filter->sink.peer = pReceivePin;
             IPin_AddRef(pReceivePin);
             hr = IMemAllocator_Commit(filter->alloc);
@@ -2273,6 +2268,8 @@ static DWORD WINAPI dispatch_thread(void *user)
 {
     struct cb_data *cbdata;
 
+    CoInitializeEx(NULL, COINIT_MULTITHREADED);
+
     pthread_mutex_lock(&cb_list_lock);
 
     while(1){
@@ -2288,6 +2285,8 @@ static DWORD WINAPI dispatch_thread(void *user)
 
     pthread_mutex_unlock(&cb_list_lock);
 
+    CoUninitialize();
+
     return 0;
 }
 
