@@ -168,7 +168,7 @@ GstFlowReturn got_data(GstPad *pad, GstObject *parent, GstBuffer *buf)
 
         gst_buffer_map(buf, &info, GST_MAP_READ);
 
-        hr = BaseOutputPinImpl_GetDeliveryBuffer((BaseOutputPin*)This->tf.ppPins[1], &sample, NULL, NULL, 0);
+        hr = BaseOutputPinImpl_GetDeliveryBuffer(&This->tf.source, &sample, NULL, NULL, 0);
         if (FAILED(hr)) {
             ERR("Could not get output buffer: %08x\n", hr);
             return GST_FLOW_FLUSHING;
@@ -205,7 +205,7 @@ GstFlowReturn got_data(GstPad *pad, GstObject *parent, GstBuffer *buf)
     IMediaSample_SetSyncPoint(sample, !GST_BUFFER_FLAG_IS_SET(buf, GST_BUFFER_FLAG_DELTA_UNIT));
     IMediaSample_SetActualDataLength(sample, gst_buffer_get_size(buf));
 
-    hr = BaseOutputPinImpl_Deliver((BaseOutputPin*)This->tf.ppPins[1], sample);
+    hr = IMemInputPin_Receive(This->tf.source.pMemInputPin, sample);
     IMediaSample_Release(sample);
     gst_buffer_unref(buf);
     if (FAILED(hr))
@@ -509,7 +509,6 @@ static HRESULT WINAPI Gstreamer_Mp3_QueryConnect(TransformFilter *iface, const A
 {
     GstTfImpl *This = (GstTfImpl*)iface;
     TRACE("%p %p\n", This, amt);
-    dump_AM_MEDIA_TYPE(amt);
 
     if ( (!IsEqualGUID(&amt->majortype, &MEDIATYPE_Audio) &&
           !IsEqualGUID(&amt->majortype, &MEDIATYPE_Stream)) ||
@@ -649,7 +648,6 @@ static HRESULT WINAPI Gstreamer_YUV_QueryConnect(TransformFilter *iface, const A
 {
     GstTfImpl *This = (GstTfImpl*)iface;
     TRACE("%p %p\n", This, amt);
-    dump_AM_MEDIA_TYPE(amt);
 
     if (!IsEqualGUID(&amt->majortype, &MEDIATYPE_Video) ||
         (!IsEqualGUID(&amt->formattype, &FORMAT_VideoInfo) &&
@@ -885,7 +883,6 @@ static HRESULT WINAPI Gstreamer_AudioConvert_QueryConnect(TransformFilter *iface
 {
     GstTfImpl *This = (GstTfImpl*)iface;
     TRACE("%p %p\n", This, amt);
-    dump_AM_MEDIA_TYPE(amt);
 
     if (!IsEqualGUID(&amt->majortype, &MEDIATYPE_Audio) ||
         !IsEqualGUID(&amt->subtype, &MEDIASUBTYPE_PCM) ||
