@@ -16,17 +16,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
 #include <stdarg.h>
-#include <stdlib.h>
-
-#ifdef HAVE_CORESERVICES_CORESERVICES_H
-#define GetCurrentThread MacGetCurrentThread
-#define LoadResource MacLoadResource
-#include <CoreServices/CoreServices.h>
-#undef GetCurrentThread
-#undef LoadResource
-#endif
 
 #include "windef.h"
 #include "winbase.h"
@@ -36,6 +26,8 @@
 #include "winhttp.h"
 #include "winreg.h"
 #include "winternl.h"
+#include "iphlpapi.h"
+#include "dhcpcsdk.h"
 #define COBJMACROS
 #include "ole2.h"
 #include "dispex.h"
@@ -382,10 +374,9 @@ static const struct object_vtbl connect_vtbl =
 
 static BOOL domain_matches(LPCWSTR server, LPCWSTR domain)
 {
-    static const WCHAR localW[] = { '<','l','o','c','a','l','>',0 };
     BOOL ret = FALSE;
 
-    if (!strcmpiW( domain, localW ) && !strchrW( server, '.' ))
+    if (!wcsicmp( domain, L"<local>" ) && !wcschr( server, '.' ))
         ret = TRUE;
     else if (*domain == '*')
     {
@@ -397,12 +388,12 @@ static BOOL domain_matches(LPCWSTR server, LPCWSTR domain)
              * the wildcard exactly.  E.g. if the wildcard is *.a.b, and the
              * hostname is www.foo.a.b, it matches, but a.b does not.
              */
-            dot = strchrW( server, '.' );
+            dot = wcschr( server, '.' );
             if (dot)
             {
-                int len = strlenW( dot + 1 );
+                int len = lstrlenW( dot + 1 );
 
-                if (len > strlenW( domain + 2 ))
+                if (len > lstrlenW( domain + 2 ))
                 {
                     LPCWSTR ptr;
 
@@ -410,8 +401,8 @@ static BOOL domain_matches(LPCWSTR server, LPCWSTR domain)
                      * could be a subdomain.  Compare the last portion of the
                      * server's domain.
                      */
-                    ptr = dot + len + 1 - strlenW( domain + 2 );
-                    if (!strcmpiW( ptr, domain + 2 ))
+                    ptr = dot + len + 1 - lstrlenW( domain + 2 );
+                    if (!wcsicmp( ptr, domain + 2 ))
                     {
                         /* This is only a match if the preceding character is
                          * a '.', i.e. that it is a matching domain.  E.g.
@@ -422,12 +413,12 @@ static BOOL domain_matches(LPCWSTR server, LPCWSTR domain)
                     }
                 }
                 else
-                    ret = !strcmpiW( dot + 1, domain + 2 );
+                    ret = !wcsicmp( dot + 1, domain + 2 );
             }
         }
     }
     else
-        ret = !strcmpiW( server, domain );
+        ret = !wcsicmp( server, domain );
     return ret;
 }
 
@@ -444,9 +435,9 @@ static BOOL should_bypass_proxy(struct session *session, LPCWSTR server)
     do {
         LPCWSTR tmp = ptr;
 
-        ptr = strchrW( ptr, ';' );
+        ptr = wcschr( ptr, ';' );
         if (!ptr)
-            ptr = strchrW( tmp, ' ' );
+            ptr = wcschr( tmp, ' ' );
         if (ptr)
         {
             if (ptr - tmp < MAX_HOST_NAME_LENGTH)
@@ -474,9 +465,9 @@ BOOL set_server_for_hostname( struct connect *connect, const WCHAR *server, INTE
     {
         LPCWSTR colon;
 
-        if ((colon = strchrW( session->proxy_server, ':' )))
+        if ((colon = wcschr( session->proxy_server, ':' )))
         {
-            if (!connect->servername || strncmpiW( connect->servername,
+            if (!connect->servername || wcsnicmp( connect->servername,
                 session->proxy_server, colon - session->proxy_server - 1 ))
             {
                 heap_free( connect->servername );
@@ -491,14 +482,14 @@ BOOL set_server_for_hostname( struct connect *connect, const WCHAR *server, INTE
                     (colon - session->proxy_server) * sizeof(WCHAR) );
                 connect->servername[colon - session->proxy_server] = 0;
                 if (*(colon + 1))
-                    connect->serverport = atoiW( colon + 1 );
+                    connect->serverport = wcstol( colon + 1, NULL, 10 );
                 else
                     connect->serverport = INTERNET_DEFAULT_PORT;
             }
         }
         else
         {
-            if (!connect->servername || strcmpiW( connect->servername,
+            if (!connect->servername || wcsicmp( connect->servername,
                 session->proxy_server ))
             {
                 heap_free( connect->servername );
@@ -641,7 +632,7 @@ static void request_destroy( struct object_header *hdr )
 static void str_to_buffer( WCHAR *buffer, const WCHAR *str, LPDWORD buflen )
 {
     int len = 0;
-    if (str) len = strlenW( str );
+    if (str) len = lstrlenW( str );
     if (buffer && *buflen > len)
     {
         if (str) memcpy( buffer, str, len * sizeof(WCHAR) );
@@ -1067,13 +1058,12 @@ static const struct object_vtbl request_vtbl =
 
 static BOOL add_accept_types_header( struct request *request, const WCHAR **types )
 {
-    static const WCHAR acceptW[] = {'A','c','c','e','p','t',0};
     static const DWORD flags = WINHTTP_ADDREQ_FLAG_ADD | WINHTTP_ADDREQ_FLAG_COALESCE_WITH_COMMA;
 
     if (!types) return TRUE;
     while (*types)
     {
-        if (!process_header( request, acceptW, *types, flags, TRUE )) return FALSE;
+        if (!process_header( request, L"Accept", *types, flags, TRUE )) return FALSE;
         types++;
     }
     return TRUE;
@@ -1081,13 +1071,13 @@ static BOOL add_accept_types_header( struct request *request, const WCHAR **type
 
 static WCHAR *get_request_path( const WCHAR *object )
 {
-    int len = object ? strlenW(object) : 0;
+    int len = object ? lstrlenW(object) : 0;
     WCHAR *p, *ret;
 
     if (!object || object[0] != '/') len++;
     if (!(p = ret = heap_alloc( (len + 1) * sizeof(WCHAR) ))) return NULL;
     if (!object || object[0] != '/') *p++ = '/';
-    if (object) strcpyW( p, object );
+    if (object) lstrcpyW( p, object );
     ret[len] = 0;
     return ret;
 }
@@ -1149,11 +1139,11 @@ HINTERNET WINAPI WinHttpOpenRequest( HINTERNET hconnect, LPCWSTR verb, LPCWSTR o
     request->receive_timeout = connect->session->receive_timeout;
     request->receive_response_timeout = connect->session->receive_response_timeout;
 
-    if (!verb || !verb[0]) verb = getW;
+    if (!verb || !verb[0]) verb = L"GET";
     if (!(request->verb = strdupW( verb ))) goto end;
     if (!(request->path = get_request_path( object ))) goto end;
 
-    if (!version || !version[0]) version = http1_1;
+    if (!version || !version[0]) version = L"HTTP/1.1";
     if (!(request->version = strdupW( version ))) goto end;
     if (!(add_accept_types_header( request, types ))) goto end;
 
@@ -1310,6 +1300,80 @@ BOOL WINAPI WinHttpSetOption( HINTERNET handle, DWORD option, LPVOID buffer, DWO
     return ret;
 }
 
+static IP_ADAPTER_ADDRESSES *get_adapters(void)
+{
+    ULONG err, size = 1024, flags = GAA_FLAG_SKIP_ANYCAST | GAA_FLAG_SKIP_MULTICAST |
+                                    GAA_FLAG_SKIP_DNS_SERVER | GAA_FLAG_SKIP_FRIENDLY_NAME;
+    IP_ADAPTER_ADDRESSES *tmp, *ret;
+
+    if (!(ret = heap_alloc( size ))) return NULL;
+    err = GetAdaptersAddresses( AF_UNSPEC, flags, NULL, ret, &size );
+    while (err == ERROR_BUFFER_OVERFLOW)
+    {
+        if (!(tmp = heap_realloc( ret, size ))) break;
+        ret = tmp;
+        err = GetAdaptersAddresses( AF_UNSPEC, flags, NULL, ret, &size );
+    }
+    if (err == ERROR_SUCCESS) return ret;
+    heap_free( ret );
+    return NULL;
+}
+
+static WCHAR *detect_autoproxyconfig_url_dhcp(void)
+{
+    IP_ADAPTER_ADDRESSES *adapters, *ptr;
+    DHCPCAPI_PARAMS_ARRAY send_params, recv_params;
+    DHCPCAPI_PARAMS param;
+    WCHAR name[MAX_ADAPTER_NAME_LENGTH + 1], *ret = NULL;
+    DWORD err, size;
+    BYTE *tmp, *buf = NULL;
+
+    if (!(adapters = get_adapters())) return NULL;
+
+    memset( &send_params, 0, sizeof(send_params) );
+    memset( &param, 0, sizeof(param) );
+    param.OptionId = OPTION_MSFT_IE_PROXY;
+    recv_params.nParams = 1;
+    recv_params.Params  = &param;
+
+    for (ptr = adapters; ptr; ptr = ptr->Next)
+    {
+        MultiByteToWideChar( CP_ACP, 0, ptr->AdapterName, -1, name, ARRAY_SIZE(name) );
+        TRACE( "adapter '%s' type %u dhcpv4 enabled %d\n", wine_dbgstr_w(name), ptr->IfType, ptr->Dhcpv4Enabled );
+
+        if (ptr->IfType == IF_TYPE_SOFTWARE_LOOPBACK) continue;
+        /* FIXME: also skip adapters where DHCP is disabled */
+
+        size = 256;
+        if (!(buf = heap_alloc( size ))) goto done;
+        err = DhcpRequestParams( DHCPCAPI_REQUEST_SYNCHRONOUS, NULL, name, NULL, send_params, recv_params,
+                                 buf, &size, NULL );
+        while (err == ERROR_MORE_DATA)
+        {
+            if (!(tmp = heap_realloc( buf, size ))) goto done;
+            buf = tmp;
+            err = DhcpRequestParams( DHCPCAPI_REQUEST_SYNCHRONOUS, NULL, name, NULL, send_params, recv_params,
+                                     buf, &size, NULL );
+        }
+        if (err == ERROR_SUCCESS && param.nBytesData)
+        {
+            int len = MultiByteToWideChar( CP_ACP, 0, (const char *)param.Data, param.nBytesData, NULL, 0 );
+            if ((ret = heap_alloc( (len + 1) * sizeof(WCHAR) )))
+            {
+                MultiByteToWideChar( CP_ACP, 0,  (const char *)param.Data, param.nBytesData, ret, len );
+                ret[len] = 0;
+            }
+            TRACE("returning %s\n", debugstr_w(ret));
+            break;
+        }
+    }
+
+done:
+    heap_free( buf );
+    heap_free( adapters );
+    return ret;
+}
+
 static char *get_computer_name( COMPUTER_NAME_FORMAT format )
 {
     char *ret;
@@ -1342,8 +1406,6 @@ static int reverse_lookup( const struct addrinfo *ai, char *hostname, size_t len
 
 static WCHAR *build_wpad_url( const char *hostname, const struct addrinfo *ai )
 {
-    static const WCHAR httpW[] = {'h','t','t','p',':','/','/',0};
-    static const WCHAR wpadW[] = {'/','w','p','a','d','.','d','a','t',0};
     char name[NI_MAXHOST];
     WCHAR *ret, *p;
     int len;
@@ -1353,59 +1415,66 @@ static WCHAR *build_wpad_url( const char *hostname, const struct addrinfo *ai )
 
     if (!reverse_lookup( ai, name, sizeof(name) )) hostname = name;
 
-    len = strlenW( httpW ) + strlen( hostname ) + strlenW( wpadW );
+    len = lstrlenW( L"http://" ) + strlen( hostname ) + lstrlenW( L"/wpad.dat" );
     if (!(ret = p = GlobalAlloc( 0, (len + 1) * sizeof(WCHAR) ))) return NULL;
-    strcpyW( p, httpW );
-    p += strlenW( httpW );
+    lstrcpyW( p, L"http://" );
+    p += lstrlenW( L"http://" );
     while (*hostname) { *p++ = *hostname++; }
-    strcpyW( p, wpadW );
+    lstrcpyW( p, L"/wpad.dat" );
     return ret;
 }
 
-static BOOL get_system_proxy_autoconfig_url( char *buf, DWORD buflen )
+static WCHAR *detect_autoproxyconfig_url_dns(void)
 {
-#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6
-    CFDictionaryRef settings = CFNetworkCopySystemProxySettings();
-    const void *ref;
-    BOOL ret = FALSE;
+    char *fqdn, *domain, *p;
+    WCHAR *ret;
 
-    if (!settings) return FALSE;
+    if (!(fqdn = get_computer_name( ComputerNamePhysicalDnsFullyQualified ))) return NULL;
+    if (!(domain = get_computer_name( ComputerNamePhysicalDnsDomain )))
+    {
+        heap_free( fqdn );
+        return NULL;
+    }
+    p = fqdn;
+    while ((p = strchr( p, '.' )) && is_domain_suffix( p + 1, domain ))
+    {
+        char *name;
+        struct addrinfo *ai;
+        int res;
 
-    if (!(ref = CFDictionaryGetValue( settings, kCFNetworkProxiesProxyAutoConfigURLString )))
-    {
-        CFRelease( settings );
-        return FALSE;
+        if (!(name = heap_alloc( sizeof("wpad") + strlen(p) )))
+        {
+            heap_free( fqdn );
+            heap_free( domain );
+            return NULL;
+        }
+        strcpy( name, "wpad" );
+        strcat( name, p );
+        res = getaddrinfo( name, NULL, NULL, &ai );
+        if (!res)
+        {
+            ret = build_wpad_url( name, ai );
+            freeaddrinfo( ai );
+            if (ret)
+            {
+                TRACE("returning %s\n", debugstr_w(ret));
+                heap_free( name );
+                break;
+            }
+        }
+       heap_free( name );
+       p++;
     }
-    if (CFStringGetCString( ref, buf, buflen, kCFStringEncodingASCII ))
-    {
-        TRACE( "returning %s\n", debugstr_a(buf) );
-        ret = TRUE;
-    }
-    CFRelease( settings );
+    heap_free( domain );
+    heap_free( fqdn );
     return ret;
-#else
-    static BOOL first = TRUE;
-    if (first)
-    {
-        FIXME( "no support on this platform\n" );
-        first = FALSE;
-    }
-    else
-        TRACE( "no support on this platform\n" );
-    return FALSE;
-#endif
 }
 
-#define INTERNET_MAX_URL_LENGTH 2084
-
 /***********************************************************************
  *          WinHttpDetectAutoProxyConfigUrl (winhttp.@)
  */
 BOOL WINAPI WinHttpDetectAutoProxyConfigUrl( DWORD flags, LPWSTR *url )
 {
-    BOOL ret = FALSE;
-    char system_url[INTERNET_MAX_URL_LENGTH + 1];
-
     TRACE("0x%08x, %p\n", flags, url);
 
     if (!flags || !url)
@@ -1413,82 +1482,27 @@ BOOL WINAPI WinHttpDetectAutoProxyConfigUrl( DWORD flags, LPWSTR *url )
         SetLastError( ERROR_INVALID_PARAMETER );
         return FALSE;
     }
-    if (get_system_proxy_autoconfig_url( system_url, sizeof(system_url) ))
-    {
-        WCHAR *urlW;
-
-        if (!(urlW = strdupAW( system_url ))) return FALSE;
-        *url = urlW;
-        SetLastError( ERROR_SUCCESS );
-        return TRUE;
-    }
+    *url = NULL;
     if (flags & WINHTTP_AUTO_DETECT_TYPE_DHCP)
     {
-        static int fixme_shown;
-        if (!fixme_shown++) FIXME("discovery via DHCP not supported\n");
+        *url = detect_autoproxyconfig_url_dhcp();
     }
     if (flags & WINHTTP_AUTO_DETECT_TYPE_DNS_A)
     {
-        char *fqdn, *domain, *p;
-
-        if (!(fqdn = get_computer_name( ComputerNamePhysicalDnsFullyQualified ))) return FALSE;
-        if (!(domain = get_computer_name( ComputerNamePhysicalDnsDomain )))
-        {
-            heap_free( fqdn );
-            return FALSE;
-        }
-        p = fqdn;
-        while ((p = strchr( p, '.' )) && is_domain_suffix( p + 1, domain ))
-        {
-            struct addrinfo *ai;
-            char *name;
-            int res;
-
-            if (!(name = heap_alloc( sizeof("wpad") + strlen(p) )))
-            {
-                heap_free( fqdn );
-                heap_free( domain );
-                return FALSE;
-            }
-            strcpy( name, "wpad" );
-            strcat( name, p );
-            res = getaddrinfo( name, NULL, NULL, &ai );
-            if (!res)
-            {
-                *url = build_wpad_url( name, ai );
-                freeaddrinfo( ai );
-                if (*url)
-                {
-                    TRACE("returning %s\n", debugstr_w(*url));
-                    heap_free( name );
-                    ret = TRUE;
-                    break;
-                }
-            }
-            heap_free( name );
-            p++;
-        }
-        heap_free( domain );
-        heap_free( fqdn );
+        if (!*url) *url = detect_autoproxyconfig_url_dns();
     }
-    if (!ret)
+    if (!*url)
     {
         SetLastError( ERROR_WINHTTP_AUTODETECTION_FAILED );
-        *url = NULL;
+        return FALSE;
     }
-    else SetLastError( ERROR_SUCCESS );
-    return ret;
+    SetLastError( ERROR_SUCCESS );
+    return TRUE;
 }
 
-static const WCHAR Connections[] = {
-    'S','o','f','t','w','a','r','e','\\',
-    'M','i','c','r','o','s','o','f','t','\\',
-    'W','i','n','d','o','w','s','\\',
-    'C','u','r','r','e','n','t','V','e','r','s','i','o','n','\\',
-    'I','n','t','e','r','n','e','t',' ','S','e','t','t','i','n','g','s','\\',
-    'C','o','n','n','e','c','t','i','o','n','s',0 };
-static const WCHAR WinHttpSettings[] = {
-    'W','i','n','H','t','t','p','S','e','t','t','i','n','g','s',0 };
+static const WCHAR path_connections[] =
+    L"Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings\\Connections";
+
 static const DWORD WINHTTP_SETTINGS_MAGIC = 0x18;
 static const DWORD WININET_SETTINGS_MAGIC = 0x46;
 static const DWORD PROXY_TYPE_DIRECT         = 1;
@@ -1520,16 +1534,16 @@ BOOL WINAPI WinHttpGetDefaultProxyConfiguration( WINHTTP_PROXY_INFO *info )
     LONG l;
     HKEY key;
     BOOL got_from_reg = FALSE, direct = TRUE;
-    char *envproxy;
+    WCHAR *envproxy;
 
     TRACE("%p\n", info);
 
-    l = RegOpenKeyExW( HKEY_LOCAL_MACHINE, Connections, 0, KEY_READ, &key );
+    l = RegOpenKeyExW( HKEY_LOCAL_MACHINE, path_connections, 0, KEY_READ, &key );
     if (!l)
     {
         DWORD type, size = 0;
 
-        l = RegQueryValueExW( key, WinHttpSettings, NULL, &type, NULL, &size );
+        l = RegQueryValueExW( key, L"WinHttpSettings", NULL, &type, NULL, &size );
         if (!l && type == REG_BINARY &&
             size >= sizeof(struct connection_settings_header) + 2 * sizeof(DWORD))
         {
@@ -1541,7 +1555,7 @@ BOOL WINAPI WinHttpGetDefaultProxyConfiguration( WINHTTP_PROXY_INFO *info )
                     (struct connection_settings_header *)buf;
                 DWORD *len = (DWORD *)(hdr + 1);
 
-                l = RegQueryValueExW( key, WinHttpSettings, NULL, NULL, buf,
+                l = RegQueryValueExW( key, L"WinHttpSettings", NULL, NULL, buf,
                     &size );
                 if (!l && hdr->magic == WINHTTP_SETTINGS_MAGIC &&
                     hdr->unknown == 0)
@@ -1596,37 +1610,30 @@ BOOL WINAPI WinHttpGetDefaultProxyConfiguration( WINHTTP_PROXY_INFO *info )
         }
         RegCloseKey( key );
     }
-    if (!got_from_reg && (envproxy = getenv( "http_proxy" )))
+    if (!got_from_reg && (envproxy = _wgetenv( L"http_proxy" )))
     {
-        char *colon, *http_proxy = NULL;
+        WCHAR *colon, *http_proxy = NULL;
 
-        if (!(colon = strchr( envproxy, ':' ))) http_proxy = envproxy;
+        if (!(colon = wcschr( envproxy, ':' ))) http_proxy = envproxy;
         else
         {
             if (*(colon + 1) == '/' && *(colon + 2) == '/')
             {
                 /* It's a scheme, check that it's http */
-                if (!strncmp( envproxy, "http://", 7 )) http_proxy = envproxy + 7;
-                else WARN("unsupported scheme in $http_proxy: %s\n", envproxy);
+                if (!wcsncmp( envproxy, L"http://", 7 )) http_proxy = envproxy + 7;
+                else WARN("unsupported scheme in $http_proxy: %s\n", debugstr_w(envproxy));
             }
             else http_proxy = envproxy;
         }
 
         if (http_proxy && http_proxy[0])
         {
-            WCHAR *http_proxyW;
-            int len;
-
-            len = MultiByteToWideChar( CP_UNIXCP, 0, http_proxy, -1, NULL, 0 );
-            if ((http_proxyW = GlobalAlloc( 0, len * sizeof(WCHAR))))
-            {
-                MultiByteToWideChar( CP_UNIXCP, 0, http_proxy, -1, http_proxyW, len );
-                direct = FALSE;
-                info->dwAccessType = WINHTTP_ACCESS_TYPE_NAMED_PROXY;
-                info->lpszProxy = http_proxyW;
-                info->lpszProxyBypass = NULL;
-                TRACE("http proxy (from environment) = %s\n", debugstr_w(info->lpszProxy));
-            }
+            direct = FALSE;
+            info->dwAccessType = WINHTTP_ACCESS_TYPE_NAMED_PROXY;
+            info->lpszProxy = GlobalAlloc( 0, (lstrlenW(http_proxy) + 1) * sizeof(WCHAR) );
+            wcscpy( info->lpszProxy, http_proxy );
+            info->lpszProxyBypass = NULL;
+            TRACE("http proxy (from environment) = %s\n", debugstr_w(info->lpszProxy));
         }
     }
     if (direct)
@@ -1644,8 +1651,6 @@ BOOL WINAPI WinHttpGetDefaultProxyConfiguration( WINHTTP_PROXY_INFO *info )
  */
 BOOL WINAPI WinHttpGetIEProxyConfigForCurrentUser( WINHTTP_CURRENT_USER_IE_PROXY_CONFIG *config )
 {
-    static const WCHAR settingsW[] =
-        {'D','e','f','a','u','l','t','C','o','n','n','e','c','t','i','o','n','S','e','t','t','i','n','g','s',0};
     HKEY hkey = NULL;
     struct connection_settings_header *hdr = NULL;
     DWORD type, offset, len, size = 0;
@@ -1661,15 +1666,15 @@ BOOL WINAPI WinHttpGetIEProxyConfigForCurrentUser( WINHTTP_CURRENT_USER_IE_PROXY
     memset( config, 0, sizeof(*config) );
     config->fAutoDetect = TRUE;
 
-    if (RegOpenKeyExW( HKEY_CURRENT_USER, Connections, 0, KEY_READ, &hkey ) ||
-        RegQueryValueExW( hkey, settingsW, NULL, &type, NULL, &size ) ||
+    if (RegOpenKeyExW( HKEY_CURRENT_USER, path_connections, 0, KEY_READ, &hkey ) ||
+        RegQueryValueExW( hkey, L"DefaultConnectionSettings", NULL, &type, NULL, &size ) ||
         type != REG_BINARY || size < sizeof(struct connection_settings_header))
     {
         ret = TRUE;
         goto done;
     }
     if (!(hdr = heap_alloc( size ))) goto done;
-    if (RegQueryValueExW( hkey, settingsW, NULL, &type, (BYTE *)hdr, &size ) ||
+    if (RegQueryValueExW( hkey, L"DefaultConnectionSettings", NULL, &type, (BYTE *)hdr, &size ) ||
         hdr->magic != WININET_SETTINGS_MAGIC)
     {
         ret = TRUE;
@@ -1758,8 +1763,7 @@ static BOOL parse_script_result( const char *result, WINHTTP_PROXY_INFO *info )
 
 static char *download_script( const WCHAR *url, DWORD *out_size )
 {
-    static const WCHAR typeW[] = {'*','/','*',0};
-    static const WCHAR *acceptW[] = {typeW, NULL};
+    static const WCHAR *acceptW[] = {L"*/*", NULL};
     HINTERNET ses, con = NULL, req = NULL;
     WCHAR *hostname;
     URL_COMPONENTSW uc;
@@ -1972,7 +1976,7 @@ BOOL WINAPI WinHttpSetDefaultProxyConfiguration( WINHTTP_PROXY_INFO *info )
         return FALSE;
     }
 
-    l = RegCreateKeyExW( HKEY_LOCAL_MACHINE, Connections, 0, NULL, 0,
+    l = RegCreateKeyExW( HKEY_LOCAL_MACHINE, path_connections, 0, NULL, 0,
         KEY_WRITE, NULL, &key, NULL );
     if (!l)
     {
@@ -1981,9 +1985,9 @@ BOOL WINAPI WinHttpSetDefaultProxyConfiguration( WINHTTP_PROXY_INFO *info )
 
         if (info->dwAccessType == WINHTTP_ACCESS_TYPE_NAMED_PROXY)
         {
-            size += strlenW( info->lpszProxy );
+            size += lstrlenW( info->lpszProxy );
             if (info->lpszProxyBypass)
-                size += strlenW( info->lpszProxyBypass );
+                size += lstrlenW( info->lpszProxyBypass );
         }
         buf = heap_alloc( size );
         if (buf)
@@ -1999,14 +2003,14 @@ BOOL WINAPI WinHttpSetDefaultProxyConfiguration( WINHTTP_PROXY_INFO *info )
                 BYTE *dst;
 
                 hdr->flags = PROXY_TYPE_PROXY;
-                *len++ = strlenW( info->lpszProxy );
+                *len++ = lstrlenW( info->lpszProxy );
                 for (dst = (BYTE *)len, src = info->lpszProxy; *src;
                     src++, dst++)
                     *dst = *src;
                 len = (DWORD *)dst;
                 if (info->lpszProxyBypass)
                 {
-                    *len++ = strlenW( info->lpszProxyBypass );
+                    *len++ = lstrlenW( info->lpszProxyBypass );
                     for (dst = (BYTE *)len, src = info->lpszProxyBypass; *src;
                         src++, dst++)
                         *dst = *src;
@@ -2020,7 +2024,7 @@ BOOL WINAPI WinHttpSetDefaultProxyConfiguration( WINHTTP_PROXY_INFO *info )
                 *len++ = 0;
                 *len++ = 0;
             }
-            l = RegSetValueExW( key, WinHttpSettings, 0, REG_BINARY, buf, size );
+            l = RegSetValueExW( key, L"WinHttpSettings", 0, REG_BINARY, buf, size );
             if (!l)
                 ret = TRUE;
             heap_free( buf );
@@ -2126,22 +2130,15 @@ BOOL WINAPI WinHttpSetTimeouts( HINTERNET handle, int resolve, int connect, int
 }
 
 static const WCHAR wkday[7][4] =
-    {{'S','u','n', 0}, {'M','o','n', 0}, {'T','u','e', 0}, {'W','e','d', 0},
-     {'T','h','u', 0}, {'F','r','i', 0}, {'S','a','t', 0}};
+    {L"Sun", L"Mon", L"Tue", L"Wed", L"Thu", L"Fri", L"Sat"};
 static const WCHAR month[12][4] =
-    {{'J','a','n', 0}, {'F','e','b', 0}, {'M','a','r', 0}, {'A','p','r', 0},
-     {'M','a','y', 0}, {'J','u','n', 0}, {'J','u','l', 0}, {'A','u','g', 0},
-     {'S','e','p', 0}, {'O','c','t', 0}, {'N','o','v', 0}, {'D','e','c', 0}};
+    {L"Jan", L"Feb", L"Mar", L"Apr", L"May", L"Jun", L"Jul", L"Aug", L"Sep", L"Oct", L"Nov", L"Dec"};
 
 /***********************************************************************
  *           WinHttpTimeFromSystemTime (WININET.@)
  */
 BOOL WINAPI WinHttpTimeFromSystemTime( const SYSTEMTIME *time, LPWSTR string )
 {
-    static const WCHAR format[] =
-        {'%','s',',',' ','%','0','2','d',' ','%','s',' ','%','4','d',' ','%','0',
-         '2','d',':','%','0','2','d',':','%','0','2','d',' ','G','M','T', 0};
-
     TRACE("%p, %p\n", time, string);
 
     if (!time || !string)
@@ -2150,7 +2147,8 @@ BOOL WINAPI WinHttpTimeFromSystemTime( const SYSTEMTIME *time, LPWSTR string )
         return FALSE;
     }
 
-    sprintfW( string, format,
+    swprintf( string, WINHTTP_TIME_FORMAT_BUFSIZE / sizeof(WCHAR),
+              L"%s, %02d %s %4d %02d:%02d:%02d GMT",
               wkday[time->wDayOfWeek],
               time->wDay,
               month[time->wMonth - 1],
@@ -2189,15 +2187,15 @@ BOOL WINAPI WinHttpTimeToSystemTime( LPCWSTR string, SYSTEMTIME *time )
 
     SetLastError( ERROR_SUCCESS );
 
-    while (*s && !isalphaW( *s )) s++;
+    while (*s && !iswalpha( *s )) s++;
     if (s[0] == '\0' || s[1] == '\0' || s[2] == '\0') return TRUE;
     time->wDayOfWeek = 7;
 
     for (i = 0; i < 7; i++)
     {
-        if (toupperW( wkday[i][0] ) == toupperW( s[0] ) &&
-            toupperW( wkday[i][1] ) == toupperW( s[1] ) &&
-            toupperW( wkday[i][2] ) == toupperW( s[2] ) )
+        if (towupper( wkday[i][0] ) == towupper( s[0] ) &&
+            towupper( wkday[i][1] ) == towupper( s[1] ) &&
+            towupper( wkday[i][2] ) == towupper( s[2] ) )
         {
             time->wDayOfWeek = i;
             break;
@@ -2205,19 +2203,19 @@ BOOL WINAPI WinHttpTimeToSystemTime( LPCWSTR string, SYSTEMTIME *time )
     }
 
     if (time->wDayOfWeek > 6) return TRUE;
-    while (*s && !isdigitW( *s )) s++;
-    time->wDay = strtolW( s, &end, 10 );
+    while (*s && !iswdigit( *s )) s++;
+    time->wDay = wcstol( s, &end, 10 );
     s = end;
 
-    while (*s && !isalphaW( *s )) s++;
+    while (*s && !iswalpha( *s )) s++;
     if (s[0] == '\0' || s[1] == '\0' || s[2] == '\0') return TRUE;
     time->wMonth = 0;
 
     for (i = 0; i < 12; i++)
     {
-        if (toupperW( month[i][0]) == toupperW( s[0] ) &&
-            toupperW( month[i][1]) == toupperW( s[1] ) &&
-            toupperW( month[i][2]) == toupperW( s[2] ) )
+        if (towupper( month[i][0]) == towupper( s[0] ) &&
+            towupper( month[i][1]) == towupper( s[1] ) &&
+            towupper( month[i][2]) == towupper( s[2] ) )
         {
             time->wMonth = i + 1;
             break;
@@ -2225,24 +2223,24 @@ BOOL WINAPI WinHttpTimeToSystemTime( LPCWSTR string, SYSTEMTIME *time )
     }
     if (time->wMonth == 0) return TRUE;
 
-    while (*s && !isdigitW( *s )) s++;
+    while (*s && !iswdigit( *s )) s++;
     if (*s == '\0') return TRUE;
-    time->wYear = strtolW( s, &end, 10 );
+    time->wYear = wcstol( s, &end, 10 );
     s = end;
 
-    while (*s && !isdigitW( *s )) s++;
+    while (*s && !iswdigit( *s )) s++;
     if (*s == '\0') return TRUE;
-    time->wHour = strtolW( s, &end, 10 );
+    time->wHour = wcstol( s, &end, 10 );
     s = end;
 
-    while (*s && !isdigitW( *s )) s++;
+    while (*s && !iswdigit( *s )) s++;
     if (*s == '\0') return TRUE;
-    time->wMinute = strtolW( s, &end, 10 );
+    time->wMinute = wcstol( s, &end, 10 );
     s = end;
 
-    while (*s && !isdigitW( *s )) s++;
+    while (*s && !iswdigit( *s )) s++;
     if (*s == '\0') return TRUE;
-    time->wSecond = strtolW( s, &end, 10 );
+    time->wSecond = wcstol( s, &end, 10 );
 
     time->wMilliseconds = 0;
     return TRUE;
